/*
 * SharedHash.h
 *
 * Shared memory hash table with long term disk storage.
 *
 * Copyright 2001, 2002 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_mod_watch_SharedHash_h__
#define __com_snert_mod_watch_SharedHash_h__	1

/*
 * This is the data that will be kept in shared memory.
 */
struct shEntry {
	char *key;
	unsigned long ifInOctets;
	unsigned long ifOutOctets;
	unsigned long ifRequests;
	unsigned long ifDocuments;
	unsigned long ifAvgRate;
	short ifActive;
};

/*
 * This statistical information will be kept in shared memory.
 */
struct shInfo {
	unsigned long found;		/* found in hash table */
	unsigned long probes;		/* number of extra probes made */
	unsigned long faults;		/* swapped to disk */
	unsigned long flushes;		/* out of shared memory */
};

/*
 * This will be kept in conventional memory.
 */
struct shTable {
	void *mutex;
	void *memory;			/* Memory header. */
	void *shared;			/* Start of shared memory block. */
	void *eshared;			/* First byte beyond the end of block.*/
	char *pathname;			/* Workdir + NAME_MAX space */
	const char *workdir;		/* The working directory. */
	struct shInfo *info;		/* Statistic information. */
	struct shEntry *array;
	int size;			/* Allocated number of entries. */
};

extern const char shLockFile[];

extern int shLock(struct shTable *table);
extern int shUnlock(struct shTable *table);
extern unsigned long shHashCode(unsigned long hash, const char *k);
extern int shContainsKey(struct shTable *table, const char *key);
extern struct shEntry *shGetLockedEntry(struct shTable *table, const char *key);
extern struct shTable *shCreate(struct pool *p, int size, const char *workdir);
extern void shDestroy(void *data);

#endif /* __com_snert_mod_watch_SharedHash_h__ */